;**********************************************************************************
;Title:			Serial interfscing with 2 X 16 LCD module source code
;Version:			1.0
;Last update:		2002.02.23
;Target MCU:		PIC16F628
;support E-mail:		mindtan@yahoo.com
;support website:		www.geocities.com/mindtan2000.html
;Register Usage:		14
;Interrupt Usage:		0
;total memory space
;after compile to hex: 	249 word (1 word = 14bit)
;Total Flash Program:	2048 X 1 word (2048 X 14 bit)
;
;Description	
;
; This program interfaces to a Hitachi compatible 2 line by 20 character display
; module. The program assembles for 4-bit data serial interface through shift
; register (74HC164) by 3 wire. Shift register CD4094 may also can be used but need 4 wire. 
; The data, commond and RS line for LCD was control by MCU through shift register. 
; Only Enable line for LCD was directly control by MCU.
;       
;
;********************************************************************************
	LIST P=16f628
      ERRORLEVEL  -302
       include <p16f628.inc>
	page
	__CONFIG _CP_OFF & _XT_OSC & _PWRTE_ON & _WDT_OFF
	
;**************************************************************************************
;Definition and address/register setting for temporary data storage. 
;*************************************************************************************
Dev_Freq		EQU		D'4000000'			; define Device Frequency is 4 MHz
DB_HI_BYTE		EQU		(HIGH ((( Dev_Freq / 4 ) * 1 / D'1000' ) / 2 ) ) + 1
LCD_INIT_DELAY	EQU		(HIGH ((( Dev_Freq / 4 ) * D'66' / D'10000' ) / 2 ) ) + 1

DISP_ON		EQU		0x00e			;define Display on value
CLR_DISP		EQU		0x001		
ENTRY_INC		EQU		0x028			;define 4 bit mode for LCD display
;----------------------------------------------
;LCD Display Commands and Control Signal names.
;----------------------------------------------
LCD_CNTL    EQU   PORTA		;set LCD control register
E		EQU	3		;LCD Enable control line
;
dly5		equ	0x035    
;
serial_port 	equ	PORTA	;set serial I/O port register
DO		equ	0		;define serial data output pin from MCU
SCK		equ	1		;define clock output pin from MCU
STR		equ	2		;define store pin output from MCU
;--------------------------
;define temporary register
;--------------------------
CHAR			EQU	0x20	;address for temporary store value will 
							;be send to LCD through shift register
TEMP			EQU	0x21	
MSD			EQU	0x22	;Temporary register, Holds Most Significant Digit of BIN to BCD conversion
LSD			EQU	0x23	;Temporary register
TEMP1			EQU	0x24	;Temporary register 
repeat		equ	0x25	;Temporary register
;
bit_count		equ	0x26	;Temporary register
rxdata		equ	0x27	;Temporary register
data_out		equ	0x28	;Temporary register
data_temp		equ	0x29	;Temporary register

Dlybyte2		equ	0x2a	;Temporary register
Dlybyte1		equ	0x2b	;Temporary register

addrMSB0		equ	0x2c	;Temporary register
addrLSB0		equ	0x2d	;Temporary register
;
;**********************************************************************************
;Begining of program
;**********************************************************************************
       org     	0x0000		; RESET vector location
       GOTO    	START          ;
;
;
START 
	call		Port_Init
	call		LCD_DELAY
	call		CMD_SEQ		;sequence to initializa the LCD
	call		Display_data0
check	
	btfss		PORTB,0		;RB0 pin low, display_data1
	call		Display_data1
	btfss		PORTB,1		;RB1 pin low, display_data
	call		Display_data
	goto		check
	

	
   
;************************************************************************************
;Initialization of hardware and Port setting
;************************************************************************************
Port_Init
      CLRF    	STATUS          ; Do initialization (Bank 0)
      bcf	INTCON,7
      clrf	PORTA
	clrf	PORTB
	movlw	0x07
	movwf	CMCON            
;
      BSF     	STATUS, RP0     ; Select Bank 1
	movlw	b'01111111'
	movwf	OPTION_REG
      CLRF    	TRISA           ; RA5 -  0 outputs
      movlw	  	b'01000011'
	movwf	  	TRISB			; RB7, 5 - 2 output, RB6, 1 - 0 input            
      BCF     	STATUS, RP0     ; Select Bank 0
	return


;******************************************************************************
; This routine takes the calculated times that the delay loop needs to
; be executed, based on the LCD_INIT_DELAY EQUate that includes the
; frequency of operation. These uses registers before they are needed to 
; store the time.
;*******************************************************************************
LCD_DELAY   
	MOVLW   	LCD_INIT_DELAY  ;
      MOVWF   	MSD             ; Use MSD and LSD Registers to Initilize LCD
      CLRF    	LSD             ;
LOOP2 
      DECFSZ  	LSD, F          ; Delay time = MSD * ((3 * 256) + 3) * Tcy
      GOTO    	LOOP2           ;
      DECFSZ  	MSD, F          ;
END_LCD_DELAY
      GOTO    	LOOP2           ;
	return

;***********************************************************************************
; Command sequence for 2 lines of 5x7 characters LCD modules
;***********************************************************************************
CMD_SEQ
	movlw		0x03
	movwf		repeat
reloop
	MOVLW 	0x28       		;8-bit high nibble xfer
      movwf		data_out
	call		serial		;output command through shift register
	call		delay_interval
	BSF   	LCD_CNTL, E     	;
	nop
      BCF   	LCD_CNTL, E     	;
	decfsz	repeat, 1
	goto		reloop

	movlw		0x16
	movwf		dly5
loopdly
	DECFSZ 	dly5, F     ; Delay time = MSD * ((3 * 256) + 3) * Tcy
      GOTO    	loopdly           
		

      MOVLW   	0x06          ; 8-bit high nibble xfer
      CALL    	SEND_CMD 
      return

;******************************************************************************
;Sub : serial
;Description : shift register serial interfacing
;******************************************************************************
serial
	movlw		h'08'
	movwf		bit_count
	bcf		serial_port, DO
	bcf		serial_port, STR
rwspiloop
	movf		data_out,w
	andlw		0x80
	movwf		data_temp
	bcf		serial_port,SCK	;start here, sck = 0
	btfss		data_temp,7		;check bit 7 equ to 1?
	bcf		serial_port,DO	;no, DO = 0
	btfsc		data_temp,7		;No, check bit 7 equ to 0?
	bsf		serial_port,DO	;no, DO = 1
	bsf		serial_port, STR	;storage register clock high
	bsf		serial_port,SCK	;sck = 1
	call		dlyus
	bcf		serial_port, STR	;storage register clock low
	rlf		data_out,7
	decfsz	bit_count,1
	goto		rwspiloop
	bcf		serial_port,SCK
	bcf		serial_port, STR	;storage register clock low
	bcf		serial_port, DO
	nop
	nop
	return


;*******************************************************************
;*SendChar - Sends character to LCD                                *
;*This routine splits the character into the upper and lower       * 
;*nibbles and sends them to the LCD, upper nibble first.           *
;*******************************************************************
SEND_CHAR
      movwf		CHAR
	MOVF    	CHAR,w
      ANDLW   	0xF0            ;Get upper nibble
	iorlw		0x01			;RS high (LCD)
	movwf		data_out
      call		serial		;Send data to LCD through shift register
      CALL    	delay_interval  ;Wait for LCD to be ready
      BSF     	LCD_CNTL, E     ;toggle E for LCD
	nop
      BCF     	LCD_CNTL, E
      SWAPF   	CHAR, w
      ANDLW   	0xF0            ;Get lower nibble
	iorlw		0x01
	movwf		data_out
      call		serial		;Send data to LCD
      BSF     	LCD_CNTL, E     ;toggle E for LCD
	nop
      BCF     	LCD_CNTL, E
      RETURN

;*******************************************************************
;* SEND_CMD - Sends command to LCD                                 *
;* This routine splits the command into the upper and lower        * 
;* nibbles and sends them to the LCD, upper nibble first.          *
;*******************************************************************
SEND_CMD
      movwf		CHAR
	MOVF    	CHAR,w
      ANDLW   	0xF0            ;Get upper nibble
	movwf		data_out
      call		serial		;Send data to LCD through shift register
	CALL    	delay_interval  ;Wait for LCD to be ready
	BSF     	LCD_CNTL,E      ; toggle E for LCD
	nop
      BCF     	LCD_CNTL,E
      SWAPF   	CHAR, w
      ANDLW   	0xF0            ;Get lower nibble
	movwf		data_out
      call		serial
      BSF     	LCD_CNTL,E      ; toggle E for LCD
	nop
      BCF     	LCD_CNTL,E
      RETURN


;****************************************************************************************
;time delay to for interval setting of LCD
;****************************************************************************************
delay_interval
	MOVLW   	0x16
      MOVWF   	MSD             ; Use MSD and LSD Registers to Initilize LCD
      CLRF    	LSD             ;
LOOP  DECFSZ  	LSD, F          ; Delay time = MSD * ((3 * 256) + 3) * Tcy
      GOTO    	LOOP           ;
      DECFSZ  	MSD, F          ;
      GOTO    	LOOP           ;
	return

dlyus				;time delay to ensure 50% duty cycle clock timing
	nop
	nop
	nop
	nop
	return


;**********************************************************************************
; Display data routine
;**********************************************************************************
Display_data0
      MOVLW   	DISP_ON         ;
      CALL    	SEND_CMD        ;
      MOVLW   	CLR_DISP        ;
      CALL    	SEND_CMD        ;
      MOVLW   	ENTRY_INC       ;
      CALL    	SEND_CMD        ;
	MOVLW		ENTRY_INC       ;
      CALL    	SEND_CMD        ;
;
      movlw   	B'10000100'     ;Address DDRam first character, first line
      call    	SEND_CMD
	
;Demonstration of the use of a table to output a message
      movlw   	0               ;Table address of start of message
dispmsg0 
      movwf   	TEMP1           ;TEMP1 holds start of message address
      call    	Table0
      andlw   	0FFh            ;Check if at end of message (zero
      btfsc   	STATUS,Z        ;returned at end)
      goto		next
      call    	SEND_CHAR       ;Display character
      movf    	TEMP1,w         ;Point to next character
      addlw   	1
      goto    	dispmsg0
next
	movlw   	B'11000010'     ;Address DDRam first character, second line
      call    	SEND_CMD
	
;Demonstration of the use of a table to output a message
      movlw   	0               ;Table address of start of message
dispmsg01 
      movwf   	TEMP1           ;TEMP1 holds start of message address
      call    	Table1
      andlw   	0FFh            ;Check if at end of message (zero
      btfsc   	STATUS,Z        ;returned at end)
      return
      call    	SEND_CHAR       ;Display character
      movf    	TEMP1,w         ;Point to next character
      addlw   	1
      goto    	dispmsg01


;**********************************************************************************
; Display first line data routine
;**********************************************************************************
Display_data
      MOVLW   	DISP_ON         ;
      CALL    	SEND_CMD        ;
      MOVLW   	CLR_DISP        ;
      CALL    	SEND_CMD        ;
      MOVLW   	ENTRY_INC       ;
      CALL    	SEND_CMD        ;
	MOVLW		ENTRY_INC       ;
      CALL    	SEND_CMD        ;
;
      movlw   	B'10000110'     ;Address DDRam first character, first line
      call    	SEND_CMD
	
;Demonstration of the use of a table to output a message
      movlw   	0               ;Table address of start of message
dispmsg 
      movwf   	TEMP1           ;TEMP1 holds start of message address
      call    	Table2
      andlw   	0FFh            ;Check if at end of message (zero
      btfsc   	STATUS,Z        ;returned at end)
      return
      call    	SEND_CHAR       ;Display character
      movf    	TEMP1,w         ;Point to next character
      addlw   	1
      goto    	dispmsg

;**********************************************************************************
; Display second line data routine
;**********************************************************************************
Display_data1
      MOVLW   	DISP_ON         ;
      CALL    	SEND_CMD        ;
      MOVLW   	CLR_DISP        ;
      CALL    	SEND_CMD        ;
      MOVLW   	ENTRY_INC       ;
      CALL    	SEND_CMD        ;
	MOVLW   	ENTRY_INC       ;
      CALL    	SEND_CMD        ;
;
      movlw   	B'11000101'     ;Address DDRam first character, second line
      call    	SEND_CMD
	
;Demonstration of the use of a table to output a message
      movlw   	0               ;Table address of start of message
dispmsg1 
      movwf   	TEMP1           ;TEMP1 holds start of message address
      call    	Table3
      andlw   	0FFh            ;Check if at end of message (zero
      btfsc   	STATUS,Z        ;returned at end)
      return
      call    	SEND_CHAR       ;Display character
      movf    	TEMP1,w         ;Point to next character
      addlw   	1
      goto    	dispmsg1




;****************************************************************************************
;Table of character will be display
;****************************************************************************************
Table0
      addwf   PCL, F          ;Jump to char pointed to in W reg
      retlw   'P'
      retlw   'r'
      retlw   'e'
      retlw   's'
      retlw   's'
      retlw   ' '
      retlw   's'
      retlw   'w'
	retlw   'i'
      retlw   't'
	retlw   'c'
	retlw   'h'
      retlw   0

Table1
      addwf   PCL, F          ;Jump to char pointed to in W reg
      retlw   'c'
      retlw   'h'
      retlw   'a'
      retlw   'n'
      retlw   'g'
      retlw   'e'
      retlw   ' '
      retlw   'd'
	retlw   'i'
      retlw   's'
	retlw   'p'
	retlw   'l'
	retlw   'a'
	retlw   'y'
      retlw   0


Table2
      addwf   PCL, F          ;Jump to char pointed to in W reg
      retlw   'f'
      retlw   'i'
      retlw   'r'
      retlw   's'
      retlw   't'
      retlw   ' '
      retlw   'l'
      retlw   'i'
	retlw   'n'
      retlw   'e'
      retlw   0

Table3
	addwf   PCL, F          	
      retlw   's'
      retlw   'e'
      retlw   'c'
      retlw   'o'
      retlw   'n'
      retlw   'd'
      retlw   ' '
      retlw   'l'
      retlw   'i'
      retlw   'n'
      retlw   'e'
      retlw   0
  

    	end




